/*
 * ReportGenerator.h
 *
 *  Created on: 1 janv. 2010
 *      Author: didier
 */

#ifndef REPORTGENERATOR_H_
#define REPORTGENERATOR_H_

#include <string>
#include <vector>

#include <Core/Core.h>
#include <Report/Report.h>

namespace ReportParser
{
	class StringReplacer;
}




class ReportGenerator
{
	public:
		typedef std::vector<int> DepthContextType;
		typedef Upp::String      StringType;
		typedef std::string      InternalStringType;

	public:
		ReportGenerator();
		virtual ~ReportGenerator();

		typedef enum
		{
			DO_NOT_LOOP = 0,
			LOOP_AGAIN = 1
		} ExecuteReturnValues;

		virtual void processHeader(const DepthContextType& ctxt ) = 0;
		virtual ExecuteReturnValues processBody(const DepthContextType& ctxt ) = 0;
		virtual void processFooter(const DepthContextType& ctxt ) = 0;

		void replaceVar(const StringType& label, const StringType& value);

		void replaceVar(const StringType& label, const char* value)
		{
			replaceVar(label, Upp::String(value));
		}

		void replaceVar(const StringType& label, char* value)
		{
			replaceVar(label, Upp::String(value));
		}

		template <class T>
		void replaceVar(const StringType& label, const T& value)
		{
			replaceVar(label, AsString(value));
		}

		
		void replaceImage(const StringType& label, const Upp::Image& inputImg, Upp::Size destSize );

		StringType getAnnotatedDoc(const StringType& templat );
		StringType generateReport(const StringType& templat);
		StringType getDocStructure(const StringType& templat );

		bool isSameContext(const DepthContextType& ctxt, int depth, int sec0=0, int sec1=0, int sec2=0, int sec3=0, int sec4=0, int sec5=0, int sec6=0, int sec7=0, int sec8=0);

		void setCurrentStringReplacer(ReportParser::StringReplacer* currReplacer) { currentReplacer = currReplacer; }
		void appendText(const InternalStringType& str) { reportResult += str; }


	private:
		ReportParser::StringReplacer* currentReplacer;
		InternalStringType reportResult;
};

#endif /* REPORTGENERATOR_H_ */
